﻿using OpcLabs.EasyOpc.DataAccess;
using OpcLabs.EasyOpc.DataAccess.OperationModel;
using System;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.IO;
using System.ServiceProcess;

namespace OPC_DAClient_Service
{
    public partial class OPC_DAClient_Service : ServiceBase
    {
        SqlConnection con = new SqlConnection(Properties.Settings.Default.connectionString);
        SqlCommand cmd;
        DAItemGroupArguments[] handleArray;

        public OPC_DAClient_Service()
        {
            InitializeComponent();

            try
            {
                con.Open();
            }
            catch (Exception exc)
            {
                EventLog.WriteEntry("Service", exc.Message, EventLogEntryType.Error);
            }

            cmd = new SqlCommand("UPDATE T_DataLive SET value=@value,time=@time WHERE ID=@ID;", con);
            cmd.Parameters.Add("@ID", SqlDbType.Int);
            cmd.Parameters.Add("@value", SqlDbType.Decimal);
            cmd.Parameters.Add("@time", SqlDbType.DateTime2);
        }

        protected override void OnStart(string[] args)
        {
            try
            {
                ReadVariables();
                easyDAClient.SubscribeMultipleItems(handleArray);
            }
            catch (Exception exc)
            {
                EventLog.WriteEntry("Service", exc.Message, EventLogEntryType.Error);
            }
        }

        protected override void OnStop()
        {
            easyDAClient.UnsubscribeAllItems();            
        }

        private void ReadVariables()
        {
            string[] rows = File.ReadAllLines(Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location), "Variables.txt"));
            EventLog.WriteEntry("Service", string.Format("{0} Variablen eingelesen.", rows.Length), EventLogEntryType.Information);

            if (rows.Length == 0)
                throw new Exception("Variablen nicht definiert.");

            handleArray = new DAItemGroupArguments[rows.Length];

            if (string.IsNullOrEmpty(Properties.Settings.Default.UrlString))
                throw new Exception("UrlString nicht definiert.");

            for (int i = 0; i < rows.Length; i++)
            {
                string[] columns = rows[i].Split(';');

                if (string.IsNullOrEmpty(columns[0]))
                    throw new Exception(string.Format("NodePath {0} nicht definiert.", i + 1));

                if (string.IsNullOrEmpty(columns[1]))
                    throw new Exception(string.Format("ItemId {0} nicht definiert.", i + 1));

                if (string.IsNullOrEmpty(columns[2]))
                    throw new Exception(string.Format("ID {0} nicht definiert.", i + 1));

                handleArray[i] = new DAItemGroupArguments();
                handleArray[i].ServerDescriptor.UrlString = "http://192.168.178.101/soap/OpcXml";

                handleArray[i].ItemDescriptor.NodePath = columns[0];
                handleArray[i].ItemDescriptor.ItemId = columns[1];

                handleArray[i].State = columns[2];
            }
        }

        private void easyDAClient_ItemChanged(object sender, EasyDAItemChangedEventArgs e)
        {
            try
            {
                if (e.Exception == null && e.Vtq != null && e.Vtq.HasValue)
                {
                    if (con.State != ConnectionState.Open)
                        con.Open();

                    cmd.Parameters["@ID"].Value = e.Arguments.State;
                    cmd.Parameters["@value"].Value = e.Vtq.Value;
                    cmd.Parameters["@time"].Value = e.Vtq.Timestamp;
                    cmd.ExecuteNonQuery();                                        
                }
                else if (e.Exception != null && Properties.Settings.Default.debug)
                    EventLog.WriteEntry("Service", e.Exception.Message, EventLogEntryType.Error);          
            }
            catch (Exception exc)
            {
                EventLog.WriteEntry("Service", exc.Message, EventLogEntryType.Error);
            }
        }
    }
}
